@@ -3,13 +3,19 @@ import functools
 import multiprocessing
 import threading
 import time
+import warnings
 from concurrent.futures import ThreadPoolExecutor
 from functools import wraps
 from unittest import TestCase
 
 import pytest
 
-from asgiref.sync import ThreadSensitiveContext, async_to_sync, sync_to_async
+from asgiref.sync import (
+    ThreadSensitiveContext,
+    async_to_sync,
+    iscoroutinefunction,
+    sync_to_async,
+)
 from asgiref.timeout import timeout
 
 
@@ -98,6 +104,18 @@ async def test_async_to_sync_fail_partial():
     )
 
 
+@pytest.mark.asyncio
+async def test_sync_to_async_raises_typeerror_for_async_callable_instance():
+    class CallableClass:
+        async def __call__(self):
+            return None
+
+    with pytest.raises(
+        TypeError, match="sync_to_async can only be applied to sync functions."
+    ):
+        sync_to_async(CallableClass())
+
+
 @pytest.mark.asyncio
 async def test_sync_to_async_decorator():
     """
@@ -365,6 +383,29 @@ def test_async_to_sync_partial():
     assert result["worked"]
 
 
+def test_async_to_sync_on_callable_object():
+    """
+    Tests async_to_sync on a callable class instance
+    """
+
+    result = {}
+
+    class CallableClass:
+        async def __call__(self, value):
+            await asyncio.sleep(0)
+            result["worked"] = True
+            return value
+
+    # Run it (without warnings)
+    with warnings.catch_warnings():
+        warnings.simplefilter("error")
+        sync_function = async_to_sync(CallableClass())
+        out = sync_function(42)
+
+    assert out == 42
+    assert result["worked"] is True
+
+
 def test_async_to_sync_method_self_attribute():
     """
     Tests async_to_sync on a method copies __self__.
@@ -397,15 +438,21 @@ def test_thread_sensitive_outside_sync():
     @async_to_sync
     async def middle():
         await inner()
+        await asyncio.create_task(inner_task())
 
-    # Inner sync function
+    # Inner sync functions
     @sync_to_async
     def inner():
         result["thread"] = threading.current_thread()
 
+    @sync_to_async
+    def inner_task():
+        result["thread2"] = threading.current_thread()
+
     # Run it
     middle()
     assert result["thread"] == threading.current_thread()
+    assert result["thread2"] == threading.current_thread()
 
 
 @pytest.mark.asyncio
@@ -603,8 +650,29 @@ def test_sync_to_async_detected_as_coroutinefunction():
     def sync_func():
         return
 
-    assert not asyncio.iscoroutinefunction(sync_to_async)
-    assert asyncio.iscoroutinefunction(sync_to_async(sync_func))
+    assert not iscoroutinefunction(sync_to_async)
+    assert iscoroutinefunction(sync_to_async(sync_func))
+
+
+async def async_process(queue):
+    queue.put(42)
+
+
+def sync_process(queue):
+    """Runs async_process synchronously"""
+    async_to_sync(async_process)(queue)
+
+
+def fork_first():
+    """Forks process before running sync_process"""
+    queue = multiprocessing.Queue()
+    fork = multiprocessing.Process(target=sync_process, args=[queue])
+    fork.start()
+    fork.join(3)
+    # Force cleanup in failed test case
+    if fork.is_alive():
+        fork.terminate()
+    return queue.get(True, 1)
 
 
 @pytest.mark.asyncio
@@ -613,26 +681,6 @@ async def test_multiprocessing():
     Tests that a forked process can use async_to_sync without it looking for
     the event loop from the parent process.
     """
-
-    test_queue = multiprocessing.Queue()
-
-    async def async_process():
-        test_queue.put(42)
-
-    def sync_process():
-        """Runs async_process synchronously"""
-        async_to_sync(async_process)()
-
-    def fork_first():
-        """Forks process before running sync_process"""
-        fork = multiprocessing.Process(target=sync_process)
-        fork.start()
-        fork.join(3)
-        # Force cleanup in failed test case
-        if fork.is_alive():
-            fork.terminate()
-        return test_queue.get(True, 1)
-
     assert await sync_to_async(fork_first)() == 42
 
 
